var Command = require('jasmine/lib/command.js');
var Jasmine = require('jasmine/lib/jasmine.js');
var Reporter = require('jasmine-terminal-reporter');

require('babel/register');

function normaliseArgs(args) {
  var opts = {
    cwd: process.cwd()
  };

  for (var i = 0; i < args.length; i += 1) {
    var arg = args[i];
    var type = typeof arg;

    if (type === 'string') {
      opts.cwd = arg;
    } else if (type === 'function') {
      opts.cb = arg;
    } else if (Array.isArray(arg)) {
      opts.args = arg;
    } else if (type === 'object') {
      opts.options = arg;
    }
  }
  return opts;
}

module.exports = function () {
  var opts = normaliseArgs(arguments);
  var jasmine = new Jasmine({ projectBaseDir: opts.cwd });
  var colour = process.argv.indexOf('--no-color') === -1;
  var includeStackTrace = opts.includeStackTrace === false ? false : true;

  jasmine.addReporter(new Reporter({
    isVerbose: opts.verbose,
    showColors: colour,
    includeStackTrace: includeStackTrace
  }));

  if ('function' === typeof opts.cb) {
    jasmine.addReporter(new CallbackReporter(opts.cb));
  }

  return new Command(opts.cwd).run(jasmine, opts.args);
};

function CallbackReporter(cb) {
  var hasFailed = false;
  this.specDone = function (result) {
    if (result.status === 'failed') {
      hasFailed = true;
    }
  };

  this.jasmineDone = function () {
    cb(hasFailed ? new Error('jasmine-babel exit code 1') : null);
  };
};
